classdef DesignNonRotHyperEllipse <Design
    % D=DesignNonRotHyperEllipse(n) creates an instance of an n-dimensional design D(x) 
    %  given by:
    %
    %       D(x)={ y in R^n with:
    %              sum_i=1^n((yi-xi)^2/xn+1^2) -1<= 0}
    %       
    %       Vol(x)=V*Prod_i=1^n(xn+i)
    %           V is the Volume of the n-dimensional unit ball
    %
    %       x in X = { x in R^(n+1) with
    %               10^-6 <= xi for i=n+1,...2n}
    properties (GetAccess=protected)
               
        dim                                     
        nrOfParams                              
        nrOfDesignConstrs=1;                     
        initValsParams                      
        
        linEqParamConstrs =[];                   
        linIneqParamConstrs =[];                    
        boundsOnParams
        
        nrOfNonlinParamEqConstrs =0;             
        nrOfNonlinParamIneqConstrs=0;            

        %volume der Einhetiskugel
        volsphere;
    end
    
    methods  %constructor
        function p=DesignNonRotHyperEllipse(dim,varargin)
            %Setzen der Eigenschaften, die sich aus dim ergeben
            p.dim=dim;
            p.nrOfParams=2*dim;
            p.initValsParams=[ones(1,dim),ones(1,dim)];
            p.boundsOnParams=[-inf*ones(1,dim),1.e-6*ones(1,dim);inf*ones(1,p.nrOfParams)];
            
            %Berechnung des volumes f�r die Einheitskugel
             if mod(dim,2)
                k=(dim-1)/2;
                p.volsphere=2*(2*pi)^k/prod(1:2:dim);
            else
                k=dim/2;
                p.volsphere=pi^k/factorial(k);
            end
            
        end
    end
    
    methods
        %volume
        function v=vol(obj,x)                     
            v=obj.volsphere*abs(prod(x(obj.dim+1:end)));
        end
        function grad_v=gradVol(obj,x)           
            temp=obj.volsphere*abs(prod(x(obj.dim+1:end)));
            grad_v=[zeros(obj.dim,1);(temp./x(obj.dim+1:end))'];
        end
        
        %design functions
        function u=evalNonlinDescrFuns(obj,x,y)                
            u=sum(((y-x(1:obj.dim))./x(obj.dim+1:end)).^2)-1;
            
        end    
        function Dy_u=DyNonlinDescrFuns(obj,x,y)               
            Dy_u=2*(y-x(1:obj.dim))./x(obj.dim+1:end).^2;
        end
        function Dx_u=DxNonlinDescrFuns(obj,x,y)
            Dx_u=[-2*(y-x(1:obj.dim))./x(obj.dim+1:end).^2,-2*((y-x(1:obj.dim)).^2)./(x(obj.dim+1:end).^3)];
        end
        function y=getInteriorPoint(obj,x)                   
            y=x(1:obj.dim);
        end 
        function Dx_y=DxInnerPoint(obj,~)
            Dx_y=[eye(obj.dim),zeros(obj.dim)];
        end
        
        function [lb,ub]=getBoundingBox(obj,x)         
            lb=x(1:obj.dim)-x(obj.dim+1:end);
            ub=x(1:obj.dim)+x(obj.dim+1:end);
        end
%                 function [Dx_lb,Dx_ub]=DxBoundingBox(obj,x)
%             
%         end

        
    end
    
        
    methods
        function y=evalTrafoFun(obj,x,z)
            m=size(z,1);           
            n=obj.dim();
            y=z*diag(x(n+1:end))+ones(m,1)*x(1:n); 
        end
        function z=evalInvTrafoFun(obj,x,y)
             m=size(y,1);
            n=obj.dim();
            temp=y-ones(m,1)*x(1:n);
            temp=temp*diag(1./x(n+1:end));
            z=temp;
        end
        function Dx_y=DxTrafoFun(obj,x,z)
            m=size(z,1);
            Dx_y=zeros(obj.dim,obj.nrOfParams,m);
            for i=1:m
                Dx_y(:,:,i)=[eye(obj.dim),diag(z(i,:))];
            end
        end
    end
    
    methods
        function []=plotDesign(obj,x)
            switch obj.dim
                case 2
                    t=0:pi/180:2*pi;
                    y1t=sin(t);
                    y2t=cos(t);
                    y1=x(3)*y1t+x(1);
                    y2=x(4)*y2t+x(2);
                    fill(y1,y2,'green');
                case 3
                    [y1t,y2t,y3t]=sphere();
                    y1=x(4)*y1t+x(1);
                    y2=x(5)*y2t+x(2);
                    y3=x(6)*y3t+x(3);
                    s = surfl(y1, y2, y3);
                    set(s, 'EdgeColor', 'none' , 'FaceColor', 'green', 'FaceAlpha', 0.75, 'FaceLighting', 'gouraud');
                    camlight;
            end
        end
    end
    
end

