classdef Problem17 < ImplicitGSIPProblem
    % p=Problem17 creates an instance of the GSIP problem:
    %
    %       min   -x1
    %
    %       s.t.  -y*x2 <=0 for all y in Y(x)
    %             10^-6 <= x1 <= 1
    %             -1 <= x2 <= 1
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               x1-y^2<= 0
    %               -1 <= y <= 1}
    
    
    % One constraint is added, such that Y(x) doesn't become empty    
    properties(GetAccess=protected)
        
        nrOfUppLevVars=2;                         
        boundsOnUppLevVars=[-1,-1;1,1];            
        nrOfSemiInfConstrs=1;                      
        initValsOfUppLevVars=[1,0];               
        
        linEqsOnUppLevVars=[];                    
        linIneqsOnUppLevVars=[-1,0,-1.e-6];       
        nrOfNonlinEqsUppLev=0;               
        nrOfNonlinIneqsUppLev=0;             
        
        
        nrOfLowLevVars=1;                         
        nrOfLowLevNonlinConstrs=1;                       
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    
    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=-x(1);
        end
        function grad_f=gradObjFun(obj,x)                   
            grad_f=[-1;0];
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)     
                                                        
            g=-y*x(2);            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)      
            n=size(y,1);
            Dx_g=[zeros(n,1),-y];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            n=size(y,1);
            Dy_g=-x(2)*ones(n,1);                
        end
               
        
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)           
            u=[x(1)-y^2];                        
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)              
            Dy_u=[-2*y];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)              
            Dx_u=[1,0];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)           
            [lb,ub] = obj.getInterval(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)    
             lb=-1;
             ub=1;
        end
                        

        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
                        [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
        
    end
    methods(Access=private)            
        
        function [lb,ub] = getInterval(obj,x)           
            lb=-sqrt(x(1));
            ub=sqrt(x(1));
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            Dx_lb=[-1/2/sqrt(x(1)),0];
            Dx_ub=[1/2/sqrt(x(1)),0];
        end

    end
    
end


